﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Selection;
using System.Linq;

namespace HeuristicLab.Analysis.FitnessLandscape {
  [Item("Random Walk", "A random walk applies a certain manipulation operation over and over.")]
  [StorableClass]
  [Creatable(CreatableAttribute.Categories.TestingAndAnalysis + CreatableAttribute.Categories.SplitToken + "1" + CreatableAttribute.Categories.OrderToken + "FLA", Priority = 301)]
  public sealed class RandomWalk : LocalAnalysis<RandomSelector> {

    [StorableConstructor]
    private RandomWalk(bool deserializing) : base(deserializing) { }
    private RandomWalk(RandomWalk original, Cloner cloner) : base(original, cloner) { }
    public RandomWalk()
      : base(new RandomSelector()) {
      SelectorParameter.Value.NumberOfSelectedSubScopesParameter.Hidden = true;
      MainLoop.SampleSizeParameter.Value = new IntValue(1);
      var trailAnalyzer = AnalyzerParameter.Value.Operators.OfType<QualityTrailMultiAnalyzer>().Single();
      var upDown = trailAnalyzer.Operators.OfType<UpDownWalkAnalyzer>().Single();
      trailAnalyzer.Operators.SetItemCheckedState(upDown, false);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RandomWalk(this, cloner);
    }
  }
}
