﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Common.Resources;
using HeuristicLab.MainForm;
using HeuristicLab.Scripting;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace HeuristicLab.OptimizationExpertSystem.Menu {
  internal class SaveCSharpScriptMenuItem : MenuItemBase {
    public override Image Image { get { return VSImageLibrary.Save; } }

    public override string Name {
      get { return "Save"; }
    }

    public override IEnumerable<string> Structure {
      get { return new[] { "Tools", "C# Script" }; }
    }

    public override int Position {
      get { return 930; }
    }

    public override string ToolTipText { get { return "Save an open script."; } }

    public override void Execute() {
      var script = (CSharpScript)((IContentView)MainForm.ActiveView).Content;
      if (!string.IsNullOrEmpty(script.Filename) && File.Exists(script.Filename)) {
        try {
          ContentManager.Save(script, script.Filename, true);
          return;
        } catch { }
      }
      using (var dialog = new SaveFileDialog()) {
        dialog.Filter = "HL Files|*.hl";
        if (dialog.ShowDialog(MainForm) != DialogResult.OK) return;
        ContentManager.Save(script, dialog.FileName, true);
      }
    }

    protected override void OnActiveViewChanged(object sender, EventArgs e) {
      base.OnActiveViewChanged(sender, e);
      var contentView = MainForm.ActiveView as IContentView;
      ToolStripItem.Enabled = (contentView != null && contentView.Content is CSharpScript);
    }
  }
}
